@@ -91,6 +91,12 @@ void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *
 		ereport(ERROR,
 				(errmsg("failed to get process info for current process")));
 
+	/* Initialize preferred master node id. This must be done before any
+	 * statement using MASTER* macros because inside the macro,
+	 * pool_get_preferred_master_node_id() is called.
+	 */
+	pool_reset_preferred_master_node_id();
+
 	/* Choose load balancing node if necessary */
 	if (pool_config->load_balance_mode)
 	{
@@ -155,9 +161,6 @@ void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *
 
 	/* Initialize previous pending message */
 	pool_pending_message_reset_previous_message();
-
-	/* Initialize preferred master node id */
-	pool_reset_preferred_master_node_id();
 }
 
 /*
@@ -230,10 +233,19 @@ void pool_set_query_in_progress(void)
  */
 void pool_unset_query_in_progress(void)
 {
+	POOL_SESSION_CONTEXT *s = pool_get_session_context(false);
+
 	ereport(DEBUG1,
 		(errmsg("session context: unsetting query in progress. DONE")));
 
-	pool_get_session_context(false)->in_progress = false;
+	s->in_progress = false;
+
+	/* Restore where_to_send map if neccessary */
+	if (s->need_to_restore_where_to_send)
+	{
+		memcpy(s->query_context->where_to_send,s->where_to_send_save, sizeof(s->where_to_send_save));
+	}
+	s->need_to_restore_where_to_send = false;
 }
 
 /*
@@ -335,6 +347,9 @@ bool pool_remove_sent_message(char kind, const char *name)
 	int i;
 	POOL_SENT_MESSAGE_LIST *msglist;
 
+	if (kind == 0 || name == NULL)
+		return false;
+
 	msglist = &pool_get_session_context(false)->message_list;
 
 	for (i = 0; i < msglist->size; i++)
@@ -563,6 +578,9 @@ POOL_SENT_MESSAGE *pool_get_sent_message(char kind, const char *name, POOL_SENT_
 
 	msglist = &pool_get_session_context(false)->message_list;
 
+	if (kind == 0 || name == NULL)
+		return NULL;
+
 	for (i = 0; i < msglist->size; i++)
 	{
 		if (msglist->sent_messages[i]->kind == kind &&
@@ -1068,6 +1086,13 @@ void pool_pending_message_query_context_dest_set(POOL_PENDING_MESSAGE* message,
 {
 	int i;
 
+	POOL_SESSION_CONTEXT *s = pool_get_session_context(false);
+
+	/* Save where_to_send map */
+	memcpy(s->where_to_send_save, query_context->where_to_send, sizeof(s->where_to_send_save));
+	s->need_to_restore_where_to_send = true;
+
+	/* Rewrite where_to_send map */
 	memset(query_context->where_to_send, 0, sizeof(query_context->where_to_send));
 
 	for (i=0;i<2;i++)
@@ -1135,8 +1160,9 @@ void pool_pending_message_add(POOL_PENDING_MESSAGE* message)
 
 	if (message->type != POOL_SYNC)
 		ereport(Elevel,
-				(errmsg("pool_pending_message_add: message type:%d message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
-						message->type, message->contents_len, message->query, message->statement, message->portal,
+				(errmsg("pool_pending_message_add: message type:%s message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
+						pool_pending_message_type_to_string(message->type),
+						message->contents_len, message->query, message->statement, message->portal,
 						message->node_ids[0], message->node_ids[1])));
 	else
 		ereport(Elevel,
@@ -1174,8 +1200,9 @@ POOL_PENDING_MESSAGE *pool_pending_message_head_message(void)
 	m = (POOL_PENDING_MESSAGE *) lfirst(cell);
 	message = copy_pending_message(m);
 	ereport(Elevel,
-			(errmsg("pool_pending_message_head_message: message type:%d message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
-					message->type, message->contents_len, message->query, message->statement, message->portal,
+			(errmsg("pool_pending_message_head_message: message type:%s message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
+					pool_pending_message_type_to_string(message->type),
+					message->contents_len, message->query, message->statement, message->portal,
 					message->node_ids[0], message->node_ids[1])));
 
 	MemoryContextSwitchTo(old_context);
@@ -1210,8 +1237,9 @@ POOL_PENDING_MESSAGE *pool_pending_message_pull_out(void)
 	m = (POOL_PENDING_MESSAGE *) lfirst(cell);
 	message = copy_pending_message(m);
 	ereport(Elevel,
-			(errmsg("pool_pending_message_pull_out: message type:%d message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
-					message->type, message->contents_len, message->query, message->statement, message->portal,
+			(errmsg("pool_pending_message_pull_out: message type:%s message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
+					pool_pending_message_type_to_string(message->type),
+					message->contents_len, message->query, message->statement, message->portal,
 					message->node_ids[0], message->node_ids[1])));
 
 	pool_pending_message_free_pending_message(m);
@@ -1357,6 +1385,71 @@ bool pool_pending_message_exists(void)
 	return list_length(session_context->pending_messages) > 0;
 }
 
+/*
+ * Convert enum pending message type to string. The returned string must not
+ * be modified or freed.
+ */
+const char *pool_pending_message_type_to_string(POOL_MESSAGE_TYPE type)
+{
+	static const char *pending_msg_string[] = {"Parse", "Bind", "Execute",
+											   "Descripbe", "Close", "Sync"};
+	if (type < 0 || type > POOL_SYNC)
+		return "unknown type";
+	return pending_msg_string[type];
+}
+
+/*
+ * Find the latest pending message having specified query context.  The
+ * returned message is a pointer to the message list. So do not free it using
+ * pool_pending_message_free_pending_message.
+ */
+POOL_PENDING_MESSAGE *pool_pending_message_find_lastest_by_query_context(POOL_QUERY_CONTEXT *qc)
+{
+	List *msgs;
+	POOL_PENDING_MESSAGE *msg;
+	int len;
+	ListCell *cell;
+
+	if (!session_context)
+	{
+		ereport(ERROR,
+				(errmsg("pool_pending_message_find_lastest_by_query_context: session context is not initialized")));
+	}
+
+	if (!session_context->pending_messages)
+		return NULL;
+
+	msgs = session_context->pending_messages;
+
+	len = list_length(msgs);
+	if (len <= 0)
+		return NULL;
+
+	ereport(DEBUG1,
+			(errmsg("pool_pending_message_find_lastest_by_query_context: num messages: %d",
+					len)));
+
+	while (len--)
+	{
+		cell = list_nth_cell(msgs, len);
+		if (cell)
+		{
+			msg = (POOL_PENDING_MESSAGE *) lfirst(cell);
+			if (msg->query_context == qc)
+			{
+				ereport(DEBUG1,
+						(errmsg("pool_pending_message_find_lastest_by_query_context: msg found. type: %s",
+								pool_pending_message_type_to_string(msg->type))));
+				return msg;
+			}
+			ereport(DEBUG1,
+					(errmsg("pool_pending_message_find_lastest_by_query_context: type: %s",
+							pool_pending_message_type_to_string(msg->type))));
+		}
+	}
+	return NULL;
+}
+
 /*
  * Dump whole pending message list
  */
