@@ -2353,7 +2353,11 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 				}
 				status = SimpleForwardToBackend(fkind, frontend, backend, len, contents);
 
-				if (pool_is_doing_extended_query_message())
+				/*
+				 * After flush message received, extended query mode should be
+				 * continued.
+				 */
+				if (fkind != 'H' && pool_is_doing_extended_query_message())
 				{
 					pool_unset_doing_extended_query_message();
 				}
@@ -3415,8 +3419,8 @@ static void pool_wait_till_ready_for_query(POOL_CONNECTION_POOL *backend)
  * Called when error response received in streaming replication mode and doing
  * extended query. Remove all pending messages and backend message buffer data
  * except POOL_SYNC pending message and ready for query.  If sync message is
- * not received yet, call ProcessFrontendResponse() to read data from
- * frontend.
+ * not received yet, continue to read data from frontend until a sync message
+ * is read.
  */
 static void pool_discard_except_sync_and_ready_for_query(POOL_CONNECTION *frontend,
 														 POOL_CONNECTION_POOL *backend)
