@@ -8,6 +8,8 @@ WHOAMI=`whoami`
 source $TESTLIBS
 TESTDIR=testdir
 
+PSQL=$PGBIN/psql
+
 for mode in s r n
 do
 	rm -fr $TESTDIR
@@ -36,12 +38,12 @@ do
 	./startall
 	wait_for_pgpool_startup
 
-	psql test <<EOF
+	$PSQL test <<EOF
 CREATE TABLE t1 (i int);
 CREATE TABLE black_t (i int);
 CREATE VIEW normal_v AS SELECT * FROM t1;
 CREATE VIEW white_v AS SELECT * FROM t1;
-SELECT pg_sleep(2);	-- Sleep for a while to make sure object creations are replicated
+
 SELECT * FROM t1;
 SELECT * FROM t1;
 SELECT * FROM black_t;
@@ -52,16 +54,11 @@ SELECT * FROM white_v;
 SELECT * FROM white_v;
 EOF
 
-	success=true
-	grep "fetched from cache" log/pgpool.log | grep t1 > /dev/null || success=false
-	grep "fetched from cache" log/pgpool.log | grep black_t > /dev/null && success=false
-	grep "fetched from cache" log/pgpool.log | grep normal_v > /dev/null && success=false
-	grep "fetched from cache" log/pgpool.log | grep white_v > /dev/null || success=false
-	if [ $success = false ];then
-		./shutdownall
-		exit 1
-	fi
-	    
+	grep "fetched from cache" log/pgpool.log | grep t1 > /dev/null || exit 1
+	grep "fetched from cache" log/pgpool.log | grep black_t > /dev/null && exit 1
+	grep "fetched from cache" log/pgpool.log | grep normal_v > /dev/null && exit 1
+	grep "fetched from cache" log/pgpool.log | grep white_v > /dev/null || exit 1
+
 	java jdbctest > result.txt 2>&1
 	cmp ../expected.txt result.txt
 	if [ $? != 0 ];then
